<!--suppress CssUnusedSymbol -->
<style>
  .blue { color: dodgerblue; }
  .op {  border-bottom: thin solid grey; font-family: monospace; color: chocolate; margin-left:25px; width: fit-content}
  .sep { height: 2px; border-width: 0; background-color: goldenrod; }
  .text { padding: 2px 3px 2px 3px; border: thin dotted white; background-color: black; color: cadetblue; margin-left:25px;  width: fit-content}
  h1 { color: chocolate; }
</style>

# Go and Python Functional Testing
This folder contains functional tests that work with a live active server and infra, 
not with mock functions.

> _**NOTE:** Some operations noted below use the `orient.sh` utility functions for ease of use, see 
> [**ORIENT.MD**](../documentation/ORIENT.md) for more information._

<hr class="sep">

## 1. Poke Tests
All poke tests should be written in Go and should be in the `poke` folder.

The poke tests are simple tests to be run within the pipeline to verify service operation,
usually against a partial environment. 

## 2. Go Functional Tests
All go functional tests should exist under the `functional` folder.

These functional tests perform more extensive testing than the poke tests 
and also require external services (not internal code mocks).

### 2.1. Local Testing with the mock hinting service
For local testing purposes, a mock hinting service ("mockhintservice") will be started which will allow hello to "work", even 
though the returned hints and timestamps will not change.

> _**NOTE:** The mock service always returns hints 0 and 2 (NEW_POLICY and NEW_CONFIG_DATA) with a
> LastUpdateTime of "Sunday, January 1, 2023 12:00:00 GMT" and LastLiveQueryTime of "Monday, August 8, 2022 12:00:00 GMT"_

1) Start up the hello infra (clean namespace suggested):
   - At the top level of the repo, use `make delete-namespaces; make local-infra-start-notail`
   - Alternately, the orient function `+infra -d -n` can be used from any directory
2) When the hello infra is running, use `k9s` to port forward the `localstack` service
3) In the same window, start up the hello app:
   - Use `make local-app-start`
   - Alternately, the orient function `+app`
4) In a new window, move to the hello repository and enter:
    <p class="op">
    ./test/run_functionals.sh -l
    </p>
   All tests should pass:
   <pre class="text">
   Ran 4 of 4 Specs in 1.129 seconds
   SUCCESS! -- 4 Passed | 0 Failed | 0 Pending | 0 Skipped
   --- PASS: TestFunctionals (1.13s)
   PASS
   ok  	hello-api/test/functional	1.361s
   </pre>


## 3. Python Functional Tests
All python functional tests should exist under the `pytest` folder.

> _**NOTE:** This folder may be removed at a future date if testing with the
> python SensorSim tool is not used._

### 3.1. SGUIDS
All python functional tests have a wavefront SGUID:
* psc:platform:ems:device-hello

This is applied to each test using the SGUID fixture:

```python
@pytest.mark.sguid(SGUID.psc_platform_ems_device_hello)
def test_01_sample(self, hc_connect):
    pass
```
